@@ -17,12 +17,25 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Fields.py,v 1.11 1999/09/02 12:57:31 gobry Exp $
+# $Id: Fields.py,v 1.12 1999/09/24 07:21:06 gobry Exp $
 
-import string, types, re, string
+import string, types, re, string, recode
 
 year_match = re.compile ('(\d\d\d\d)')
 
+formatter_cache = {}
+
+def get_formatter (format):
+    format = string.lower (format)
+    
+    if formatter_cache.has_key (format):
+        ft = formatter_cache [format]
+    else:
+        ft = recode.recode ("latin1.." + format)
+        formatter_cache [format] = ft
+
+    return ft
+
 class Access:
     """ virtual class used to check if instance attributes have been
     modified """
@@ -50,7 +63,6 @@ class Access:
         self.__dict__ ['_modified_'] = 1
         return
 
-        
 class Author (Access):
     """ Fine description of an author """
     
@@ -63,6 +75,7 @@ class Author (Access):
             self.lineage   = None
 
             blocs = string.split (copy, ',')
+            
             if len (blocs) == 1:
                 if strict:
                     # strict parsing, the whole block is the last name
@@ -82,13 +95,20 @@ class Author (Access):
             elif len (blocs) == 2:
                 self.last  = string.strip (blocs [0])
                 self.first = string.strip (blocs [1])
+                
+            elif len (blocs) == 3:
+                self.last    = string.strip (blocs [0])
+                self.lineage = string.strip (blocs [1])
+                self.first   = string.strip (blocs [2])
+
             else:
                 self.last  = copy
                 self.first = None
 
             # cleanup
-            if self.last == '': self.last = None
-            if self.first == '': self.first = None
+            if self.last == '':    self.last    = None
+            if self.first == '':   self.first   = None
+            if self.lineage == '': self.lineage = None
         else:
             if copy:
                 def clean_entry (f):
@@ -114,6 +134,14 @@ class Author (Access):
 
         Access.__init__ (self)
         return
+
+    def format (self, fmt):
+        """ Returns the fields in a given format """
+        ft = get_formatter (fmt)
+            
+        return (ft (self.honorific), ft (self.first),
+                ft (self.last), ft (self.lineage))
+
     
     def __str__ (self):
         """ Returns textual representation """
@@ -124,15 +152,15 @@ class Author (Access):
             if self.honorific: text = text + " " + self.honorific
             if self.first:     text = text + " " + self.first
             if self.last:      text = text + " " + self.last
-            if self.lineage:   text = text + " " + self.lineage
+            if self.lineage:   text = text + ", " + self.lineage
             self.text = text [1:]
             if not changed: self.clean ()
             
         return self.text
 
     def __repr__ (self):
-        return "Author ((%s, %s, %s, %s))" % (self.honorific, self.first,
-                                              self.last, self.lineage)
+        return "Author ((%s, %s, %s, %s))" % (`self.honorific`, `self.first`,
+                                              `self.last`, `self.lineage`)
 
 
     def match (self, regex):
@@ -140,7 +168,7 @@ class Author (Access):
         return regex.search (str (self))
 
 
-    def initials (self):
+    def initials (self, format = None):
         """ Extract initials from a first name """
 
         total = []
@@ -153,17 +181,28 @@ class Author (Access):
                 
             total.append (string.join (list, '-'))
             
-        return string.join (total, ' ')
+        text = string.join (total, ' ')
+        if format:
+            ft = get_formatter (format)
+            text = ft (text)
+
+        return text
 
     def __cmp__ (self, other):
         """ field comparison """
         
-        r = cmp (string.lower (self.last), string.lower (other.last))
+        r = cmp (self.last, other.last)
         if r != 0: return r
 
         r = cmp (self.first, other.first)
         if r != 0: return r
 
+        r = cmp (self.lineage, other.lineage)
+        if r != 0: return r
+        
+        r = cmp (self.honorific, other.honorific)
+        if r != 0: return r
+
         return 0
 
 
@@ -301,7 +340,25 @@ class Date (Access):
             
         return self.text
 
+    def format (self, fmt):
+        """ Returns the fields in a given format """
+        ft = get_formatter (fmt)
 
+        if self.year:
+            year = ft (str (self.year))
+        else: year = None
+
+        if self.month:
+            month = ft (str (self.month))
+        else: month = None
+
+        if self.day:
+            day = ft (str (self.day))
+        else: day = None
+
+        return year, month, day
+
+    
     def __repr__ (self):
         return str (self)
 
@@ -337,3 +394,9 @@ class Text (Access):
     def __cmp__ (self, other):
         return cmp (self.text, str (other))
 
+
+    def format (self, fmt):
+        """ Returns the fields in a given format """
+        ft = get_formatter (fmt)
+
+        return ft (self.text)
