@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Entry.py,v 1.9 1999/09/15 09:29:26 gobry Exp $
+# $Id: Entry.py,v 1.10 1999/09/24 07:18:34 gobry Exp $
 
 from Pyblio.Base import *
 import string, re
@@ -37,11 +37,11 @@ regular_id    = re.compile ("^\w[\d\w:_-]+$")
 class NativeEditor (GtkDialog):
     """ Implements a dialog that display a simple edition box for native entry """
     
-    def __init__ (self, entry):
+    def __init__ (self, entry, text = None):
         
-        self.entry = entry
+        self.entry    = entry
         self.modified = None
-        self.native = None
+        self.native   = None
 
         # do we have our own mainloop ?
         self.running  = 0
@@ -75,11 +75,15 @@ class NativeEditor (GtkDialog):
         
         self.vbox.pack_start (scrolled)
 
-        if self.entry.entry.has_key (self.entry.field):
-            self.original = self.entry.entry.get_native (self.entry.field)
+        if text:
+            self.original = text
             self.editor.insert_defaults (self.original)
         else:
-            self.original = None
+            if self.entry.entry.has_key (self.entry.field):
+                self.original = self.entry.entry.get_native (self.entry.field)
+                self.editor.insert_defaults (self.original)
+            else:
+                self.original = None
             
         scrolled.show_all ()
         return
@@ -101,7 +105,7 @@ class NativeEditor (GtkDialog):
 
         # get the text corresponding to the entry
         self.modified = self.entry.entry [self.entry.field]
-
+        
         # reset the old version (we are not supposed to set it now)
         if self.original is not None:
             self.entry.entry.set_native (self.entry.field, self.original)
@@ -119,9 +123,16 @@ class NativeEditor (GtkDialog):
         
         mainloop ()
         return self.modified
-    
+
+
 class BasicEditor (GtkHBox):
-    def __init__ (self, entry, field, has_native = 1):
+    def __init__ (self,
+                  entry, field,
+                  value = None, loss = 0, has_native = 1):
+
+        """ Edit a given field in a given Entry. It is possible to specify
+        the current aspect of the edition box """
+        
         GtkHBox.__init__ (self, FALSE, 5)
 
         self.entry = entry
@@ -137,12 +148,15 @@ class BasicEditor (GtkHBox):
 
         # internal variable to avoid modification notice during self.fill ()
         self.__in_fill = 0
-        
-        if entry.has_key (self.field):
+
+        # Note the previous value of the field
+        if entry.has_key (self.field) and \
+           entry.is_personal (self.field):
+            
             self.value, loss = entry.text (self.field)
         else:
             self.value, loss = None, 0
-
+            
         # add native editor if we have the native property
         if has_native:
             if loss:
@@ -173,6 +187,8 @@ class BasicEditor (GtkHBox):
         return
     
     def set_editable (self, val):
+        """ make the entry field editable or not """
+        
         self.editor.set_editable (val)
         self.button.set_sensitive (val)
         
@@ -180,7 +196,7 @@ class BasicEditor (GtkHBox):
         return
         
     def edit_native (self, * arg):
-        n = NativeEditor (self)
+        n = NativeEditor (self, self.native)
         
         # was the entry modified ?
         text = n.run_and_close ()
@@ -195,9 +211,12 @@ class BasicEditor (GtkHBox):
     
     
     def fill (self, text = None):
-        self.__in_fill = 1
+        """ Fill the field with a given Field value """
         
-        text = text or self.value
+        self.__in_fill = 1
+
+        if text is None:
+            text = self.value
         
         if text is None:
             self.editor.set_text ('')
@@ -209,6 +228,11 @@ class BasicEditor (GtkHBox):
 
     
     def update (self):
+        """ Update the real entry according to the modifications """
+
+        # entry not editable -> nothing to update
+        if not self.editable: return 0
+        
         new = self.get ()
 
         # entry has been natively modified
@@ -232,6 +256,8 @@ class BasicEditor (GtkHBox):
 
 
     def get (self):
+        """ Create a Field from the edition box """
+        
         text = string.strip (self.editor.get_text ())
         
         if text == '': return None
@@ -241,8 +267,8 @@ class BasicEditor (GtkHBox):
     
 class TextEditor (BasicEditor):
     
-    def __init__ (self, entry, field, has_native = 1):
-        BasicEditor.__init__ (self, entry, field, has_native)
+    def __init__ (self, entry, field, value = None, loss = 0, has_native = 1):
+        BasicEditor.__init__ (self, entry, field, value, loss, has_native)
         
         scrolled = GtkScrolledWindow ()
         scrolled.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
@@ -255,11 +281,13 @@ class TextEditor (BasicEditor):
         
         scrolled.add (self.editor)
         self.pack_start (scrolled, TRUE, TRUE)
-        self.fill ()
+        
+        self.fill (value)
         return
 
     def fill (self, text = None):
-        text = text or self.value
+        if text is None:
+            text = self.value
         
         self.editor.freeze ()
         self.editor.delete_text (0, -1)
@@ -279,8 +307,8 @@ class TextEditor (BasicEditor):
 
 class AuthorEditor (BasicEditor):
     
-    def __init__ (self, entry, field, has_native = 1):
-        BasicEditor.__init__ (self, entry, field, has_native)
+    def __init__ (self, entry, field, value = None, loss = 0, has_native = 1):
+        BasicEditor.__init__ (self, entry, field, value, loss, has_native)
 
         scrolled = GtkScrolledWindow ()
         scrolled.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
@@ -293,7 +321,7 @@ class AuthorEditor (BasicEditor):
         
         scrolled.add (self.editor)
         self.pack_start (scrolled, TRUE, TRUE)
-        self.fill ()
+        self.fill (value)
         return
 
     def fill (self, text = None):
@@ -309,7 +337,8 @@ class AuthorEditor (BasicEditor):
 
             return ret
 
-        text = text or self.value
+        if text is None:
+            text = self.value
         
         self.editor.freeze ()
         self.editor.delete_text (0, -1)
@@ -335,28 +364,28 @@ class AuthorEditor (BasicEditor):
     
 class FieldEditor (BasicEditor):
     
-    def __init__ (self, entry, field, has_native = 1):
-        BasicEditor.__init__ (self, entry, field, has_native)
+    def __init__ (self, entry, field, value = None, loss = 0, has_native = 1):
+        BasicEditor.__init__ (self, entry, field, value, loss, has_native)
         
         self.editor = GtkEntry ()
         self.editor.connect ("changed", self.modified_cb)
         self.editor.set_editable (TRUE)
         self.pack_start (self.editor, TRUE, TRUE)
-        self.fill ()
+        self.fill (value)
         return
 
 
 
 class DateEditor (BasicEditor):
     
-    def __init__ (self, entry, field, has_native = 1):
-        BasicEditor.__init__ (self, entry, field, has_native)
+    def __init__ (self, entry, field, value = None, loss = 0, has_native = 1):
+        BasicEditor.__init__ (self, entry, field, value, loss, has_native)
         
         self.editor = GtkEntry ()
         self.editor.connect ("changed", self.modified_cb)
         self.editor.set_editable (TRUE)
         self.pack_start (self.editor, TRUE, TRUE)
-        self.fill ()
+        self.fill (value)
         return
 
     def get (self):
@@ -378,6 +407,9 @@ class FieldGroup:
         self.dict [key] = value
         return
 
+    def keys (self):
+        return self.dict.keys ()
+    
     def __len__ (self):
         return len (self.dict.keys ())
 
@@ -439,13 +471,14 @@ class FieldGroup:
 class Entry (GtkVBox):
     """ Entry displayer """
     
-    def __init__ (self, entry, properties, acc):
+    def __init__ (self, entry, properties, acc, database):
         GtkVBox.__init__ (self, FALSE)
 
         self.entry       = entry
         self.properties  = properties
         self.changed     = 0
         self.accelerator = acc
+        self.database    = database
         
         ent = entry.type
 
@@ -548,6 +581,16 @@ class Entry (GtkVBox):
         button.add_accelerator ("clicked", self.accelerator,
                                 GDK.Escape, 0, 0)
 
+        # fill in the data for the notebook
+        self.current_data = {}
+        
+        for k in self.entry.keys ():
+            perso       = self.entry.is_personal (k)
+            value, loss = self.entry.text (k)
+            
+            self.current_data [k] = [value, loss, perso]
+            
+        # Create the notebook
         self.create_notebook (ent)
         return
 
@@ -559,12 +602,70 @@ class Entry (GtkVBox):
         return
     
     def dnd_drag_data_received (self, w, context, x, y, data, info, time):
+        name = data.data
+
+        # remove the is_personal flag everywhere
+        try:
+            crossref = self.database [Base.Key (self.entry.key.base, name)]
+        except KeyError:
+            d = GnomeErrorDialog ("Cross-Reference `%s' is unexistant in the database" % cr)
+            d.run_and_close ()
+            return
+        
         self.crossref.set_text (data.data)
+
+        for k in crossref.keys ():
+            # unless the entry already provides something, overwrite
+            # the field
+            
+            if self.mandatory.has_key (k):
+                value = self.mandatory [k].get ()
+            elif self.optional.has_key (k):
+                value = self.optional [k].get ()
+            elif self.extra.has_key (k):
+                value = self.extra [k].get ()
+            else:
+                value = None
+
+            if value is None:
+                value, loss = crossref.text (k)
+                edit        = 0
+            else:
+                loss = 0
+                edit = 1
+                
+            self.current_data [k] = (value, loss, edit)
+
+
+        # Refresh the whole stuff
+        self.remove (self.notebook)
+        self.create_notebook (self.entry.type)
         return
 
 
     def remove_crossref (self, * arg):
+        name = self.crossref.get_text ()
+        
+        try:
+            crossref = self.database [Base.Key (self.entry.key.base, name)]
+        except KeyError:
+            d = GnomeErrorDialog ("Cross-Reference `%s' is unexistant in the database" % cr)
+            d.run_and_close ()
+            return
+
+        # set the is_personal flag everywhere
+        for k in crossref.keys ():
+            if self.current_data.has_key (k):
+                value, loss, edit = self.current_data [k]
+
+            if not edit:
+                self.current_data [k] = ["", 0, 1]
+
         self.crossref.set_text ("")
+
+        # Refresh the whole stuff
+        self.remove (self.notebook)
+        self.create_notebook (self.entry.type)
         return
     
     def has_property (self, prop):
@@ -587,27 +688,34 @@ class Entry (GtkVBox):
 
         native = self.has_property ('native')
         
-        def get_field (entry, field, native = native):
+        def get_field (entry, field, native = native, self = self):
             type = FieldsInfo.fieldinfo (field).type
 
+            lc = string.lower (field)
+            
+            if self.current_data.has_key (lc):
+                value, loss, perso = self.current_data [lc]
+            else:
+                value, loss, perso = None, 0, 1
+
             if type == FieldsInfo.WidgetText: 
-                tmp = TextEditor (entry, field, native)
+                tmp = TextEditor (entry, field, value, loss, native)
 
             elif type == FieldsInfo.WidgetEntry:
-                tmp = FieldEditor (entry, field, native)
+                tmp = FieldEditor (entry, field, value, loss, native)
 
             elif type == FieldsInfo.WidgetAuthor:
-                tmp = AuthorEditor (entry, field, native)
+                tmp = AuthorEditor (entry, field, value, loss, native)
 
             elif type == FieldsInfo.WidgetDate:
-                tmp = DateEditor (entry, field, native)
+                tmp = DateEditor (entry, field, value, loss, native)
 
             else:
-                tmp = TextEditor (entry, field, native)
+                tmp = TextEditor (entry, field, value, loss, native)
 
             tmp.show ()
 
-            if (tmp.value is not None) and not entry.is_personal (string.lower (field)):
+            if (value is not None) and not perso:
                 # for cross-referenced fields
                 tmp.set_editable (FALSE)
                 
@@ -645,6 +753,31 @@ class Entry (GtkVBox):
         type = Types.getentry (type)
         self.entry.type = type
 
+        # get the changed values
+        for k in self.mandatory.keys ():
+            value       = self.mandatory [k].get ()
+
+            if self.current_data.has_key (k):
+                self.current_data [k] [0] = value
+            else:
+                self.current_data [k] = [value, 0, 1]
+
+        for k in self.optional.keys ():
+            value       = self.optional [k].get ()
+
+            if self.current_data.has_key (k):
+                self.current_data [k] [0] = value
+            else:
+                self.current_data [k] = [value, 0, 1]
+
+        for k in self.extra.keys ():
+            value       = self.extra [k].get ()
+
+            if self.current_data.has_key (k):
+                self.current_data [k] [0] = value
+            else:
+                self.current_data [k] = [value, 0, 1]
+                
         # Refresh the whole stuff
         self.remove (self.notebook)
         self.create_notebook (type)
@@ -653,13 +786,18 @@ class Entry (GtkVBox):
         return
 
     def extra_field (self, button, widget):
-        text = string.strip (widget.gtk_entry ().get_text ())
+        text = string.lower (string.strip (widget.gtk_entry ().get_text ()))
         
         if not regular_field.search (text):
             d = GnomeErrorDialog ("Invalid field name `%s'" % text)
             d.run_and_close ()
             return
 
+        if self.entry.has_key (text):
+            d = GnomeErrorDialog ("Key `%s' already defined" % text)
+            d.run_and_close ()
+            return
+            
         self.entry [text] = 'new entry'
         
         # Refresh the whole stuff
@@ -674,8 +812,9 @@ class EntryDialog (GtkDialog):
     def __init__ (self, entry, database, callback = None, user = None):
 
         GtkDialog.__init__ (self)
-        self.set_policy (TRUE, TRUE, TRUE)
         
+        self.set_policy (TRUE, TRUE, TRUE)
+
         self.set_title ('Edit')
         self.connect("delete_event", self.close)
 
@@ -686,7 +825,7 @@ class EntryDialog (GtkDialog):
         acc = GtkAccelGroup ()
         self.add_accel_group (acc)
 
-        self.entry = Entry (entry, database.properties, acc)
+        self.entry = Entry (entry, database.properties, acc, database)
         
         apply_b = GnomeStockButton (STOCK_BUTTON_APPLY)
         apply_b.connect ("clicked", self.apply)
@@ -737,15 +876,21 @@ class EntryDialog (GtkDialog):
         
         oldcr = self.entry.entry.crossref
         if oldcr: oldcr = oldcr.key
-        
+
         if oldcr != cr:
             changed = 1
-            try:
-                self.entry.entry.crossref = self.database [Base.Key (self.entry.entry.key.base, cr)]
-            except KeyError:
-                d = GnomeErrorDialog ("Cross-Reference `%s' is unexistant in the database" % cr)
-                d.run_and_close ()
-                return
+            
+            if cr is None:
+                # remove the previous crossref
+                self.entry.entry.crossref = None
+            else:
+                # add a new crossref
+                try:
+                    self.entry.entry.crossref = self.database [Base.Key (self.entry.entry.key.base, cr)]
+                except KeyError:
+                    d = GnomeErrorDialog ("Cross-Reference `%s' is unexistant in the database" % cr)
+                    d.run_and_close ()
+                    return
                 
         if changed and self.callback:
             self.callback (self.entry.entry, self.user)
@@ -756,3 +901,4 @@ class EntryDialog (GtkDialog):
     def close (self, *arg):
         self.destroy ()
         return
+
