@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Pybliographic.py,v 1.19 1999/09/15 09:29:26 gobry Exp $
+# $Id: Pybliographic.py,v 1.20 1999/09/24 07:18:35 gobry Exp $
 
 import string, os, urlparse
 
@@ -27,7 +27,7 @@ from gnome import config
 from gtk import *
 import GtkExtra, GDK, GTK
 
-from Pyblio import Open, Types, Base, Config
+from Pyblio import Open, Types, Base, Config, Autoload
 
 from Pyblio.GnomeUI import Utils
 
@@ -80,8 +80,9 @@ class URLFileSelection (GtkFileSelection):
         menu = GtkMenu ()
         self.menu.set_menu (menu)
         
-        liste = Open.available ()
-
+        liste = Autoload.available ("format")
+        liste.sort ()
+        
         if has_auto:
             Utils.popup_add (menu, " - Auto - ", self.menu_select, None)
             self.type = None
@@ -137,6 +138,95 @@ class URLFileSelection (GtkFileSelection):
         
         return self.ret
 
+
+class FormatFileSelection (GtkFileSelection):
+    
+    def __init__(self, title = "Format"):
+        
+        GtkFileSelection.__init__(self)
+        self.set_title (title)
+        
+        self.connect("destroy", self.quit)
+        self.connect("delete_event", self.quit)
+        self.cancel_button.connect('clicked', self.quit)
+        self.ok_button.connect('clicked', self.ok_cb)
+
+        grab_add (self)
+
+        self.ret = None
+        
+        vbox = self.main_vbox
+        
+        # type selector
+        hbox = GtkHBox ()
+        hbox.set_spacing (5)
+        hbox.pack_start (GtkLabel ("Formatting Style:"),
+                         expand = FALSE, fill = FALSE)
+        self.type_menu = GtkOptionMenu ()
+        hbox.pack_start (self.type_menu)
+        vbox.pack_start (hbox, expand = FALSE, fill = FALSE)
+
+        # menu content
+        menu = GtkMenu ()
+        self.type_menu.set_menu (menu)
+        
+        liste = Autoload.available ("style")
+        liste.sort ()
+        
+        for avail in liste:
+            Utils.popup_add (menu, avail, self.menu_select, avail)
+
+        self.type_menu.set_history (0)
+        self.type = liste [0]
+        
+        # type selector
+        hbox = GtkHBox ()
+        hbox.set_spacing (5)
+        hbox.pack_start (GtkLabel ("Output type:"),
+                         expand = FALSE, fill = FALSE)
+        self.output_menu = GtkOptionMenu ()
+        hbox.pack_start (self.output_menu)
+        vbox.pack_start (hbox, expand = FALSE, fill = FALSE)
+
+        # menu content
+        menu = GtkMenu ()
+        self.output_menu.set_menu (menu)
+        
+        liste = Autoload.available ("output")
+        liste.sort ()
+        
+        for avail in liste:
+            Utils.popup_add (menu, avail, self.output_menu_select, avail)
+
+        self.output_menu.set_history (0)
+        self.output = liste [0]
+        return
+
+    def menu_select (self, widget, selection):
+        self.type = selection
+        return
+
+    def output_menu_select (self, widget, selection):
+        self.output = selection
+        return
+
+    def quit(self, *args):
+        self.hide()
+        self.destroy()
+        mainquit()
+        return
+    
+    def ok_cb(self, b):
+        self.ret = self.get_filename()
+        
+        self.quit()
+        return
+    
+    def run (self):
+        self.show_all ()
+        mainloop ()
+        
+        return self.ret
     
 class Pybliographic (GnomeApp):
     """ Main Class defining a Pybliographic window """
@@ -188,6 +278,11 @@ class Pybliographic (GnomeApp):
             UIINFO_ITEM ('Sort by Type', None, self.sort_by_type),
             ]
 
+        format_menu = [
+            UIINFO_ITEM_STOCK('Format', None, self.format_entries, STOCK_MENU_SAVE),
+            UIINFO_ITEM_STOCK('Format As...', None, self.format_entries_as, STOCK_MENU_SAVE_AS),
+            ]
+        
         config_menu = [
             UIINFO_ITEM_STOCK('Preferences...', None, self.configure_prefs,
                               STOCK_MENU_PREF),
@@ -197,6 +292,7 @@ class Pybliographic (GnomeApp):
             UIINFO_SUBTREE('File', file_menu),
             UIINFO_SUBTREE('Edit', edit_menu),
             UIINFO_SUBTREE('Configure', config_menu),
+            UIINFO_SUBTREE('Formatting', format_menu),
             UIINFO_SUBTREE('Help', help_menu)
             ]
         
@@ -272,6 +368,7 @@ class Pybliographic (GnomeApp):
 
         self.search = None
         self.opened_files = []
+        self.format = None
         
         files = list (config.get_vector ("Pybliographer/Base/History="))
         for f in files:
@@ -352,6 +449,8 @@ class Pybliographic (GnomeApp):
             if arg == 0: self.modified = 0
             return
 
+        was_modified = self.modified
+        
         if self.modified:
             d = GnomeQuestionDialog ("Database is modified.\n" +
                                      "Discard changes ?", callback)
@@ -361,13 +460,20 @@ class Pybliographic (GnomeApp):
 
         try:
             self.database.set (Open.bibnew (file, type))
-            
+
+            if self.search:
+                self.search.create_root_item (self.database)
+                
         except IOError, err:
             Utils.error_dialog ('Creation error', err)
+            
+            # recover modification status
+            self.modified = was_modified
             return
 
         self.filename = file
         self.type     = type
+        self.format   = None
         
         self.modified = 0
         self.update_status ()
@@ -396,7 +502,10 @@ class Pybliographic (GnomeApp):
         try:
             self.database.set (Open.bibopen (file, how))
             Utils.set_cursor (self, 'normal')
-            
+
+            if self.search:
+                self.search.create_root_item (self.database)
+                
         except IOError, err:
             Utils.set_cursor (self, 'normal')
 
@@ -407,6 +516,8 @@ class Pybliographic (GnomeApp):
         self.type     = self.database.data.id
 
         self.modified = 0
+        self.format   = None
+        
         self.update_status ()
 
         self.add_opened_file (file)
@@ -704,6 +815,7 @@ class Pybliographic (GnomeApp):
 
             self.display.thaw ()
 
+        return
             
     # --------------------------------------------------
 
@@ -720,3 +832,43 @@ class Pybliographic (GnomeApp):
              self.database.select_row (self.database.focus_row, 0)
         return
     
+    # --------------------------------------------------
+
+    def format_entries (self, * arg):
+        if self.database.data is None: 
+            d = GnomeErrorDialog ("Please open a database first")
+            d.run_and_close ()
+            return
+        
+        if not self.format:
+            self.format_entries_as ()
+            return
+
+        try:
+            fh = open (self.format [0], 'w')
+        except IOError, err:
+            d = GnomeErrorDialog ("Can't save file:\n%s" % msg)
+            d.run_and_close ()
+            return
+
+        style  = Autoload.get_by_name ("style",  self.format [1])
+        output = Autoload.get_by_name ("output", self.format [2])
+
+        formatter = output.data (fh)
+        
+        Utils.set_cursor (self, 'clock')
+        style.data ("Bibliography", formatter, self.database.data)
+        Utils.set_cursor (self, 'normal')
+
+        fh.close ()
+        return
+    
+    def format_entries_as (self, * arg):
+        win = FormatFileSelection ()
+        win.set_filename (self.directory)
+
+        file = win.run ()
+        if file:
+            self.format = (file, win.type, win.output)
+            self.format_entries ()
+        return
