@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Open.py,v 1.16 1999/09/02 12:57:31 gobry Exp $
+# $Id: Open.py,v 1.17 1999/09/24 07:21:10 gobry Exp $
 
 
 from types import *
@@ -27,34 +27,8 @@ from Pyblio import Autoload
 import urlparse, urllib, traceback, os, sys, tempfile, string
 
 
-# ----- Generic opening -----
-
-OpeningMethods = {}
-ListedMethods = []
-
 __urlhandles = []
 
-def available ():
-	liste = ListedMethods + Autoload.available ()
-	hash  = {}
-	def _mapper (x, h = hash):
-		h [x] = 1
-		return 1
-	
-	map (_mapper, liste)
-	liste = hash.keys ()
-	liste.sort ()
-	
-	return liste
-
-def register (name, properties):
-	""
-
-	if not OpeningMethods.has_key (string.lower (name)):
-		ListedMethods.append (name)
-	
-	OpeningMethods [string.lower (name)] = properties
-
 
 Pyblio.Help.register ('bibopen', """
 Syntax: database = bibopen (source)
@@ -73,15 +47,23 @@ One can apply the following commands on the output of bibopen :
 """)
 
 
-def get (entity, method):
+def get_by_name (entity, method):
 	""" returns a specific field of the given entity """
 
-	entity = string.lower (entity)
+	meth = Autoload.get_by_name ("format", entity)
 	
-	if OpeningMethods.has_key (entity):
-		meth = OpeningMethods [entity]
-		if meth.has_key (method):
-			return meth [method]
+	if meth and meth.data.has_key (method):
+		return meth.data [method]
+
+	return None
+
+def get_by_regexp (entity, method):
+	""" returns a specific field of the given entity """
+
+	meth = Autoload.get_by_regexp ("format", entity)
+	
+	if meth and meth.data.has_key (method):
+		return meth.data [method]
 
 	return None
 
@@ -93,18 +75,24 @@ def bibopen (entity, how = None):
 		base = None
 
 		if how == None:
+			method = Autoload.available ("format")
+			
 			for method in ListedMethods:
-				opener = get (method, 'open')
+				opener = get_by_name (method, 'open')
 				if opener:
 					base = opener (url, 1)
-					if base is not None: break
+					if base is not None:
+						return base
+			return None
+		
+		opener = get_by_name (how, 'open')
+		
+		if opener:
+			base = opener (url, 0)
 		else:
-			opener = get (how, 'open')
-			if opener:
-				base = opener (url, 0)
-			else:
-				raise IOError, \
-				      "method `%s' provides no opener" % how
+			raise IOError, \
+			      "method `%s' provides no opener" % how
+		
 		return base
 	
 	# Consider the reference as an URL
@@ -117,9 +105,9 @@ def bibopen (entity, how = None):
 
 	# eventually load a new module
 	if how is None:
-		Autoload.check_by_url (url)
-	else:
-		Autoload.check_by_name (how)
+		handler = Autoload.get_by_regexp ("format", urlparse.urlunparse (url))
+		if handler:
+			how = handler.name
 	
 	base = simple_try (url, how)
 	
@@ -169,8 +157,7 @@ def bibwrite (entity, out = None, how = None):
 		entity.foreach (simplewrite, out)
 		return
 
-	Autoload.check_by_name (how)
-	writer = get (how, 'write')
+	writer = get_by_name (how, 'write')
 	
 	if writer is None:
 		raise IOError, "type `%s' does not specify write method" % how
@@ -186,7 +173,7 @@ Creates a new bibliographic database of a given type
 
 def bibnew (name, type = None):
 
-	opener = get (type, 'new')
+	opener = get_by_name (type, 'new')
 	
 	if opener is None:
 		if os.path.exists (name):
