@@ -18,7 +18,7 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
- $Id: bibtex.h,v 1.25 1999/08/27 12:36:25 gobry Exp $
+ $Id: bibtex.h,v 1.26 1999/09/24 07:15:45 gobry Exp $
 */
 
 #ifndef __bibtex_h__
@@ -247,6 +247,7 @@ extern "C" {
 					  gboolean content);
 
     BibtexStruct * bibtex_struct_copy    (BibtexStruct * source);
+    void           bibtex_struct_display (BibtexStruct * source);
     BibtexStruct * bibtex_struct_flatten (BibtexStruct * source);
     BibtexStruct * bibtex_struct_append  (BibtexStruct *, BibtexStruct *);
 
@@ -260,6 +261,10 @@ extern "C" {
 
     gchar *       bibtex_struct_as_bibtex (BibtexStruct * s);
 
+    gchar *       bibtex_struct_as_latex  (BibtexStruct * s,
+					   BibtexFieldType type,
+					   GHashTable * dico);
+
     BibtexField * bibtex_struct_as_field  (BibtexStruct * s, 
 					   BibtexFieldType type);
 
