@@ -18,7 +18,7 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
- $Id: bibtexmodule.c,v 1.8 1999/08/31 13:08:50 gobry Exp $
+ $Id: bibtexmodule.c,v 1.9 1999/09/24 07:16:20 gobry Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -190,8 +190,6 @@ bib_expand (PyObject * self, PyObject * args) {
 	break;
 
     case BIBTEX_AUTHOR:
-/*  	g_message ("author list with %d authors (None=%p)",  */
-/*  		   field->field.author->len, Py_None); */
 
 	liste = PyList_New (field->field.author->len);
 
@@ -208,10 +206,8 @@ bib_expand (PyObject * self, PyObject * args) {
 
 	    if (author->first) {
 		auth [1] = PyString_FromString (author->first);
-/*  		g_message ("first is %s", author->first); */
 	    }
 	    else {
-/*  		g_message ("first is none"); */
 		auth [1] = Py_None; 
 		Py_INCREF (Py_None);
 	    }
@@ -278,6 +274,35 @@ bib_get_native (PyObject * self, PyObject * args) {
     return tmp;
 }
 
+static PyObject *
+bib_get_latex (PyObject * self, PyObject * args) {
+    PyObject * tmp;
+    BibtexField * field;
+    PyBibtexField_Object * field_obj;
+    BibtexFieldType type;
+    BibtexSource * file;
+    PyBibtexSource_Object * file_obj;
+    gchar * text;
+    int i;
+
+    if (! PyArg_ParseTuple(args, "O!O!i:get_latex", 
+			   &PyBibtexSource_Type, & file_obj, 
+			   & PyBibtexField_Type, & field_obj,
+			   & type
+			   ))
+	return NULL;
+
+    field = field_obj->obj;
+    file  = file_obj->obj;
+
+    text = bibtex_struct_as_latex (field->structure,
+				   type, file->table);
+    tmp = Py_BuildValue("s", text); 
+    g_free (text);
+
+    return tmp;
+}
+
 static PyObject *
 bib_set_native (PyObject * self, PyObject * args) {
     PyObject * tmp;
@@ -301,7 +326,8 @@ bib_set_native (PyObject * self, PyObject * args) {
     to_parse = g_strdup_printf ("@preamble{%s}", text);
 
     if (! bibtex_source_string (source, "internal string", to_parse)) {
-	PyErr_SetString (PyExc_IOError, "can't create internal string");
+	PyErr_SetString (PyExc_IOError, 
+			 "can't create internal string for parsing");
 	return NULL;
     }
 
@@ -597,6 +623,7 @@ static PyMethodDef bibtexMeth [] = {
     { "get_offset", bib_get_offset, METH_VARARGS },
     { "expand", bib_expand, METH_VARARGS },
     { "get_native", bib_get_native, METH_VARARGS },
+    { "get_latex", bib_get_latex, METH_VARARGS },
     { "set_native", bib_set_native, METH_VARARGS },
     { "reverse", bib_reverse, METH_VARARGS },
     { "get_dict", bib_get_dict, METH_VARARGS },
